==> spirit_hub-0.1.1.6.tar.gz: spirit_hub-0.1.1.6/app/controllers/concerns/datatablable.rb <==
# frozen_string_literal: true

module Datatablable
  extend ActiveSupport::Concern

  def update_shown_columns
    display_option = find_display_option
    authorize(display_option, policy_class: SpiritHub::DisplayOptionPolicy)

    default_column_names = default_columns.keys.map(&:to_sym)
    display_option.hidden_columns[display_option_page_name] = default_column_names - params[:columns].map(&:to_sym)
    display_option.save

    @open_columns_modal = true
    find_models
    flash.now[:notice] = 'Modifié les colonnes affichées'

    render turbo_stream: [
      turbo_stream.replace(:model_table, partial: table_partial),
      turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
    ]
  end

  def update_view_type
    display_option = find_display_option
    authorize(display_option, policy_class: SpiritHub::DisplayOptionPolicy)

    display_option.view_types[display_option_page_name] = params[:view_type] || []
    display_option.save

    find_models
    flash.now[:notice] = 'Modifié le type de vue'

    render turbo_stream: [
      turbo_stream.replace(:model_table, partial: table_partial),
      turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
    ]
  end

  def set_columns
    @all_columns = default_columns
    default_column_names = default_columns.keys
    column_to_hide = Array(@user.display_option.hidden_columns&.dig(display_option_page_name))
    @user_column_names = default_column_names - column_to_hide
    @columns = default_columns.slice(*@user_column_names.map(&:to_sym))
  end

  def set_view_type
    @view_type = @user.display_option.view_types&.dig(display_option_page_name) || 'table'
  end

  private

  def controller_name
    self.class.name.demodulize.sub('Controller', '').underscore
  end

  def default_columns
    raise 'You must implement this method in your controller'
  end

  def display_option_page_name
    "#{controller_name}_page".to_sym
  end

  def table_partial
    "spirit_hub/#{controller_name}/table"
  end

  def find_models_page(scope, options = {})
    @sort_by = params.fetch(:sort_by, options[:default_sort_by]).to_s
    @sort_direction = params.fetch(:sort_direction, 'asc').to_s
    @page = params.fetch(:page, 1).to_i
    @per_page = params.fetch(:per_page, 15).to_i

    models = scope.order_by(@sort_by => @sort_direction).skip((@page - 1) * @per_page).limit(@per_page)
    @models_count = models.count
    @total_count = scope.count

    setup_pagination_data
    set_columns
    set_view_type
    set_user_favorites
    models
  end

  def setup_pagination_data
    @per_page_options = [5, 10, 15, 20, 25, 50, 100]
    @total_pages = (@models_count / @per_page.to_f).ceil
    @next_page = @total_pages > @page
    @previous_page = @page > 1

    @start_index = [(@page - 1) * @per_page + 1, @total_count].min
    @end_index = [@start_index + @per_page - 1, @total_count].min

    @range_start = [@page - 2, 1].max
    @range_end = [@range_start + 4, @total_pages].min
    @range_start = [@range_end - 4, 1].max
    @show_dot_start = @range_start > 1
    @show_dot_end = @range_end < @total_pages
  end

  def find_display_option
    policy_scope(SpiritHub::DisplayOption, policy_scope_class: SpiritHub::DisplayOptionPolicy::Scope).find_by(user: @user)
  end
end

==> spirit_hub-1.0.0.5.tar.gz: spirit_hub-1.0.0.5/app/controllers/concerns/datatablable.rb <==
# frozen_string_literal: true

module Datatablable
  extend ActiveSupport::Concern

  def update_shown_columns
    display_option = find_display_option
    authorize(display_option)

    set_columns
    hidden_columns = @all_columns.keys.map(&:to_sym) - params[:columns].map(&:to_sym)
    display_option.hidden_columns[controller_name] = hidden_columns
    display_option.save

    @open_columns_modal = true
    find_models
    flash.now[:notice] = 'Modifié les colonnes affichées'

    render turbo_stream: [
      turbo_stream.replace(:model_table, partial: table_partial),
      turbo_stream.prepend(:flash_messages, FlashMessagesComponent.new(flash))
    ]
  end

  def update_view_type
    display_option = find_display_option
    authorize(display_option)

    display_option.view_types[controller_name] = params[:view_type] || []
    display_option.save

    find_models
    flash.now[:notice] = 'Modifié le type de vue'

    render turbo_stream: [
      turbo_stream.replace(:model_table, partial: table_partial),
      turbo_stream.prepend(:flash_messages, FlashMessagesComponent.new(flash))
    ]
  end

  private

  def table_partial
    "spirit_hub/#{controller_name}/table"
  end

  def find_models_page(scope, options = {})
    @sort_by = params.fetch(:sort_by, options[:default_sort_by]).to_s
    @sort_direction = params.fetch(:sort_direction, 'asc').to_s
    @page = params.fetch(:page, 1).to_i
    @per_page = params.fetch(:per_page, 15).to_i

    models = scope.order_by(@sort_by => @sort_direction).skip((@page - 1) * @per_page).limit(@per_page)
    @models_count = models.count
    @total_count = scope.count

    setup_pagination_data
    set_columns
    set_view_type
    set_user_favorite_tagset_ids
    models
  end

  def setup_pagination_data
    @per_page_options = [5, 10, 15, 20, 25, 50, 100]
    @total_pages = (@models_count / @per_page.to_f).ceil
    @next_page = @total_pages > @page
    @previous_page = @page > 1

    @start_index = [(@page - 1) * @per_page + 1, @total_count].min
    @end_index = [@start_index + @per_page - 1, @total_count].min

    @range_start = [@page - 2, 1].max
    @range_end = [@range_start + 4, @total_pages].min
    @range_start = [@range_end - 4, 1].max
    @show_dot_start = @range_start > 1
    @show_dot_end = @range_end < @total_pages
  end

  def find_display_option
    policy_scope(SpiritHub::DisplayOption, policy_scope_class: SpiritHub::DisplayOptionPolicy::Scope).find_by(user: @user)
  end
end
